@@ -58,10 +58,15 @@ hamzafv = {
    {a="%'%_I", b="إI"},
    -- taḫfīfu 'l-hamza
    {a="'u'([^uaiUAI])", b="أU%1"},
-   {a="'i'([^uaiUAI])", b="إI%1"},   
+   {a="'i'([^uaiUAI])", b="إI%1"},
+   {a="([wf]a)%-\'([^uaiUAIl][^%-])", b="%1أْ%2"},
+   {a="^u'([^uaiUAI])", b="اU%1"},
+   {a="([^uaiUAIYN][%s%(%[%<])u'([^uaiUAI])", b="%1اU%2"},
+   {a="^i'([^uaiUAI])", b="اI%1"},
+   {a="([^uaiUAIYN][%s%(%[%<])i'([^uaiUAI])", b="%1اI%2"},
    -- madda (historic writing below)
    {a="'a'([^uaiUAI])", b="آ%1"},
-   {a="([^uiUI])\'a?A([%_%^%.]?[%`%'btjghxdrzsfqklmnywAY])", b="%1آ%2"},
+   {a="([^uiyUI])\'a?A([%_%^%.]?[%`%'btjghxdrzsfqklmnywAY])", b="%1آ%2"},
    {a="^\'a?A([%_%^%.]?[%`%'btjghxdrzsfqklmnywAY])", b="آ%1"},
    {a="(%W)\'a?A([%_%^%.]?[%`%'btjghxdrzsfqklmnywAY])", b="%1آ%2"},
    {a="(A)(')(uN?%p?)$", b="aآء%3"},
@@ -90,15 +95,15 @@ hamzafv = {
    {a="([%s%(%[%<%-])(')([ua])", b="%1أ%3"},
    {a="([%s%(%[%<%-])(')(i)", b="%1إ%3"},
    -- final
-   -- ^say'aN and .zim'aN are special orthographies
-   {a="(%^sa%.?[yY])(\"?%|?)(%')(aN)", b="%1%2ئ%4"},
-   {a="(.zi?m)(%')(aN)", b="%1ئ%3"},
-   -- hamzah + pr. suffix: the carrier passes into yāʾ before ī so
-   -- take out the following two lines:
-   -- {a="(')(I%p?)$", b="إ%2"}, -- affix pr. 1st person sg.
-   -- {a="(')(I)(%p?%s)", b="إ%2%3"}, -- affix pr. 1st person sg.
-   {a="([^uai])(')([uai]N?%p?)$", b="%1ء%3"},
-   {a="([^uai])(')([uai]N?)(%p?%s)", b="%1ء%3%4"},
+   -- mi'aT is special orthography (unlike ^say'aN and .zim'aN):
+   -- {a="(%^sa%.?[yY])(\"?%|?)(%')(aN)", b="%1%2ئ%4"}, --new
+   -- {a="(.zi?m)(%')(aN)", b="%1ئ%3"}, --new
+   {a="(mi)(%')(a[Tt])", b="%1ائ%3"},
+   {a="(mi)(%')(aN%_?[AY])", b="%1أ%3"},
+   -- final hamzah is on the line after a letter of prolongation or a
+   -- consonant with sukūn
+   {a="([^Auai])(')(\"?[uai]N?)(%p?)$", b="%1ء%3%4"}, --new
+   {a="([^Auai])(')(\"?[uai]N?)(%p?%s)", b="%1ء%3%4"},
 -- u
    {a="(u)(')([uai]N?%p?)$", b="%1ؤ%3"},
    {a="(u)(')([uai]N?)(%p?%s)", b="%1ؤ%3%4"},
@@ -124,8 +129,8 @@ hamzafv = {
    {a="(i)(')(%p?%s)", b="%1ئْ%3"},
 --
    -- middle
-   {a="(U)(')", b="%1ء"},
-   {a="([Iy])(')", b="%1ئ"},
+   {a="([UIwy])(')", b="%1ء"}, --new
+   -- {a="([Iy])(')", b="%1ئ"}, --included in the above line
    {a="([^uai])(')([uU])", b="%1ؤ%3"},
    {a="([^uai])(')(%_?[aAY])", b="%1أ%3"},
    {a="([^uai])(')([iI])", b="%1ئ%3"},
@@ -179,10 +184,15 @@ hamzafveasy = { -- differences marked below with 'easy'
    {a="%'%_I", b="إI"},
    -- taḫfīfu 'l-hamza
    {a="'u'([^uaiUAI])", b="أU%1"},
-   {a="'i'([^uaiUAI])", b="إI%1"},   
+   {a="'i'([^uaiUAI])", b="إI%1"},
+   {a="([wf]a)%-\'([^uaiUAIl][^%-])", b="%1أْ%2"},
+   {a="^u'([^uaiUAI])", b="اU%1"},
+   {a="([^uaiUAIYN][%s%(%[%<])u'([^uaiUAI])", b="%1اU%2"},
+   {a="^i'([^uaiUAI])", b="اI%1"},
+   {a="([^uaiUAIYN][%s%(%[%<])i'([^uaiUAI])", b="%1اI%2"},
    -- madda (historic writing below)
    {a="'a'([^uaiUAI])", b="آ%1"},
-   {a="([^uiUI])\'a?A([%_%^%.]?[%`%'btjghxdrzsfqklmnywAY])", b="%1آ%2"},
+   {a="([^uiyUI])\'a?A([%_%^%.]?[%`%'btjghxdrzsfqklmnywAY])", b="%1آ%2"},
    {a="^\'a?A([%_%^%.]?[%`%'btjghxdrzsfqklmnywAY])", b="آ%1"},
    {a="(%W)\'a?A([%_%^%.]?[%`%'btjghxdrzsfqklmnywAY])", b="%1آ%2"},
    --easy (begin)
@@ -213,15 +223,25 @@ hamzafveasy = { -- differences marked below with 'easy'
    {a="([%s%(%[%<%-])(')([ua])", b="%1أ%3"},
    {a="([%s%(%[%<%-])(')(i)", b="%1إ%3"},
    -- final
-   -- ^say'aN and .zim'aN are special orthographies
-   {a="(%^sa%.?[yY])(\"?%|?)(%')(aN)", b="%1%2ئ%4"},
-   {a="(.zi?m)(%')(aN)", b="%1ئ%3"},
-   -- hamzah + pr. suffix: the carrier passes into yāʾ before ī so
-   -- take out the following two lines:
-   -- {a="(')(I%p?)$", b="إ%2"}, -- affix pr. 1st person sg.
-   -- {a="(')(I)(%p?%s)", b="إ%2%3"}, -- affix pr. 1st person sg.
-   {a="([^uai])(')([uai]N?%p?)$", b="%1ء%3"},
-   {a="([^uai])(')([uai]N?)(%p?%s)", b="%1ء%3%4"},
+   -- mi'aT is special orthography (unlike ^say'aN and .zim'aN)
+   -- {a="(%^sa%.?[yY])(\"?%|?)(%')(aN)", b="%1%2ئ%4"}, --new
+   -- {a="(.zi?m)(%')(aN)", b="%1ئ%3"}, --new
+   {a="(mi)(%')(a[Tt])", b="%1ائ%3"},
+   {a="(mi)(%')(aN%_?[AY])", b="%1أ%3"},
+   -- easy (begin)
+   -- The Munjid says that such words as radI'aN do not have the
+   -- hamzah alone on the line, so take out the following two lines
+   -- (final hamzah is on the line after a letter of prolongation or a
+   -- consonant with sukūn)
+   -- {a="([^Auai])(')(\"?[uai]N?)(%p?)$", b="%1ء%3%4"}, --new
+   -- {a="([^Auai])(')(\"?[uai]N?)(%p?%s)", b="%1ء%3%4"},
+   {a="([^Auai])(')(\"?aN)(%p?)$", b="%1ئ%3%4"}, --new
+   {a="([^Auai])(')(\"?aN)(%p?%s)", b="%1ئ%3%4"}, --new
+   {a="([^uai])(')(\"?a)(%p?)$", b="%1ء%3%4"}, --new
+   {a="([^uai])(')(\"?a)(%p?%s)", b="%1ء%3%4"}, --new
+   {a="([^uai])(')(\"?[ui]N?)(%p?)$", b="%1ء%3%4"}, --new
+   {a="([^uai])(')(\"?[ui]N?)(%p?%s)", b="%1ء%3%4"}, --new
+   --easy (end)
 -- u
    {a="(u)(')([uai]N?%p?)$", b="%1ؤ%3"},
    {a="(u)(')([uai]N?)(%p?%s)", b="%1ؤ%3%4"},
@@ -247,8 +267,8 @@ hamzafveasy = { -- differences marked below with 'easy'
    {a="(i)(')(%p?%s)", b="%1ئْ%3"},
 --
    -- middle
-   {a="(U)(')", b="%1ء"},
-   {a="([Iy])(')", b="%1ئ"},
+   {a="([Uw])(')", b="%1ء"}, --new
+   {a="([Iy])(')", b="%1ئ"}, --easy
    {a="([^uai])(')([uU])", b="%1ؤ%3"},
    {a="([^uai])(')(%_?[aAY])", b="%1أ%3"},
    {a="([^uai])(')([iI])", b="%1ئ%3"},
@@ -287,7 +307,7 @@ tanwinfv = { -- with assimilations (\SetArbDflt*)
    {a="%-?(aN)(_A)(%s)([rlmnwy])", b="ًى%3%4%4"},
    {a="%-?(aN)(Y)(%s)([rlmnwy])", b="ًى%3%4%4"},
    {a="(T)%-?(aN)(%s)([rlmnwy])", b="%1ً%3%4%4"},
-   {a="(ء)%-?(aN)(%s)([rlmnwy])", b="%1ً%3%4%4"},
+   {a="(ء)%-?(aN)(%s)([rlmnwy])", b="%1%2%3%4%4"}, --new
    {a="([^TA])%-?(aN)(%s)([rlmnwy])", b="%1ًا%3%4%4"},
    {a="%-?(iNI?)(%s)([rlmnwy])", b="ٍ%2%3%3"},
    -- assimilations (end)
@@ -299,7 +319,7 @@ tanwinfv = { -- with assimilations (\SetArbDflt*)
    {a="%-?(\"aN)(_A)", b="ى"},
    {a="%-?(\"aN)(Y)", b="ى"},
    {a="(T)%-?(\"aN)", b="%1"},
-   {a="(ء)%-?(\"aN)", b="%1"},
+   {a="([اآ])(ء)%-?(\"aN)", b="%1%2"}, --new
    {a="([^TA])%-?(\"aN)", b="%1ا"},
    {a="%-?(\"iNI?)", b=""},
    -- quoted tanwīn (end)
@@ -308,7 +328,7 @@ tanwinfv = { -- with assimilations (\SetArbDflt*)
    {a="%-?(aN)(_A)", b="ًى"},
    {a="%-?(aN)(Y)", b="ًى"},
    {a="(T)%-?(aN)", b="%1ً"},
-   {a="(ء)%-?(aN)", b="%1ً"},
+   {a="([اآ])(ء)%-?(aN)", b="%1%2ً"}, --new
    {a="([^TA])%-?(aN)", b="%1ًا"},
    {a="%-?(iNI?)", b="ٍ"}
 }
@@ -334,7 +354,7 @@ tanwinfveasy = { -- no assimilations (see below)
 --easy   {a="%-?(aN)(_A)(%s)([rlmnwy])", b="ًى%3%4%4"},
 --easy   {a="%-?(aN)(Y)(%s)([rlmnwy])", b="ًى%3%4%4"},
 --easy   {a="(T)%-?(aN)(%s)([rlmnwy])", b="%1ً%3%4%4"},
---easy   {a="(ء)%-?(aN)(%s)([rlmnwy])", b="%1ً%3%4%4"},
+--easy   {a="(ء)%-?(aN)(%s)([rlmnwy])", b="%1%2%3%4%4"}, --new
 --easy   {a="([^TA])%-?(aN)(%s)([rlmnwy])", b="%1ًا%3%4%4"},
 --easy   {a="%-?(iNI?)(%s)([rlmnwy])", b="ٍ%2%3%3"},
    -- assimilations (end)
@@ -346,7 +366,7 @@ tanwinfveasy = { -- no assimilations (see below)
    {a="%-?(\"aN)(_A)", b="ى"},
    {a="%-?(\"aN)(Y)", b="ى"},
    {a="(T)%-?(\"aN)", b="%1"},
-   {a="(ء)%-?(\"aN)", b="%1"},
+   {a="([اآ])(ء)%-?(\"aN)", b="%1%2"}, --new
    {a="([^TA])%-?(\"aN)", b="%1ا"},
    {a="%-?(\"iNI?)", b=""},
    -- quoted tanwīn (end)
@@ -355,7 +375,7 @@ tanwinfveasy = { -- no assimilations (see below)
    {a="%-?(aN)(_A)", b="ًى"},
    {a="%-?(aN)(Y)", b="ًى"},
    {a="(T)%-?(aN)", b="%1ً"},
-   {a="(ء)%-?(aN)", b="%1ً"},
+   {a="([اآ])(ء)%-?(aN)", b="%1%2ً"}, --new
    {a="([^TA])%-?(aN)", b="%1ًا"},
    {a="%-?(iNI?)", b="ٍ"}
 }
@@ -520,7 +540,7 @@ trigraphsfveasy = { -- trigraphs or more (see 'easy' tag below for the diffs)
 }
 
 digraphsfvidgham = {
-   -- iʿrāb: straight double quote must be discarded
+   -- ʾiʿrāb: straight double quote must be discarded
    {a="(%-)(\"?[UI]na)(%p?%s)", b="%2%3"},
    {a="(%-)(\"?[UI]na)(%p?)$", b="%2%3"},
    {a="(%-)(\"?At[ui])(%p?%s)", b="%2%3"},
@@ -544,7 +564,7 @@ digraphsfvidgham = {
    {a="(ay)(%s)([%(%[%|%<]?)(\"?[uai])", b="%1i%2%3ٱ"}, --p
    -- hyphen + initial alif without hamza
    --   {a="([uai]%-)(\"?[uai])", b="%1ٱ"},
-   {a="([uai]%-)(\"?[uai])([%^%_%.%`]?)(%a)", b="%1ٱ%3%4"},
+   {a="([uai]%-)(\"?[uai])([%^%_%.%`]?)([%aأإؤئ])", b="%1ٱ%3%4"},
    -- initial alif without hamza (next 4 lines)
    {a="(o[%S]-)([uaiUAIY])(o)(\"?[uai])", b="ٱ"},
    {a="([%_]?[uaiUAIY])(%s[%(%[%|%<]?)(\"?[uai])", b="%1%2ٱ"}, --p
@@ -671,7 +691,7 @@ digraphsfv = {
    {a="(ay)(%s)([%(%[%|%<]?)(\"?[uai])", b="%1i%2%3ٱ"}, --p
    -- hyphen + initial alif without hamza
    --   {a="([uai]%-)(\"?[uai])", b="%1ٱ"},
-   {a="([uai]%-)(\"?[uai])([%^%_%.%`]?)(%a)", b="%1ٱ%3%4"},
+   {a="([uai]%-)(\"?[uai])([%^%_%.%`]?)([%aأإؤئ])", b="%1ٱ%3%4"},
    -- initial alif without hamza (next 4 lines)
    {a="(o[%S]-)([uaiUAIY])(o)(\"?[uai])", b="ٱ"},
    {a="([%_]?[uaiUAIY])(%s[%(%[%|%<]?)(\"?[uai])", b="%1%2ٱ"}, --p
@@ -774,7 +794,7 @@ digraphsfv = {
 }
 
 digraphsfveasy = { -- see the differences under 'easy' marker below
-   -- iʿrāb: straight double quote must be discarded
+   -- ʾiʿrāb: straight double quote must be discarded
    {a="(%-)(\"?[UI]na)(%p?%s)", b="%2%3"},
    {a="(%-)(\"?[UI]na)(%p?)$", b="%2%3"},
    {a="(%-)(\"?At[ui])(%p?%s)", b="%2%3"},
@@ -798,7 +818,7 @@ digraphsfveasy = { -- see the differences under 'easy' marker below
    {a="(ay)(%s)([%(%[%|%<]?)(\"?[uai])", b="%1i%2%3ٱ"}, --p
    -- hyphen + initial alif without hamza
    -- {a="([uai]%-)(\"?[uai])", b="%1ٱ"},
-   {a="([uai]%-)(\"?[uai])([%^%_%.%`]?)(%a)", b="%1ٱ%3%4"},
+   {a="([uai]%-)(\"?[uai])([%^%_%.%`]?)([%aأإؤئ])", b="%1ٱ%3%4"},
    -- initial alif without hamza (next 4 lines)
    {a="(o[%S]-)([uaiUAIY])(o)(\"?[uai])", b="ٱ"},
    {a="([%_]?[uaiUAIY])(%s[%(%[%|%<]?)(\"?[uai])", b="%1%2ٱ"}, --p
