@@ -47,7 +47,7 @@ tanwinnv = {
    {a="%-?(\"aN)(_A)", b="ًى"},
    {a="%-?(\"aN)(Y)", b="ًى"},
    {a="(T)%-?(\"aN)", b="%1ً"},
-   {a="(ء)%-?(\"aN)", b="%1ً"},
+   {a="([اآ])(ء)%-?(\"aN)", b="%1%2ً"}, --new
    {a="([^TA])%-?(\"aN)", b="%1ًا"},
    {a="%-?(\"iNI?)", b="ٍ"},
    -- "quoted" tanwīn (end)
@@ -56,7 +56,7 @@ tanwinnv = {
    {a="%-?(aN)(_A)", b="ى"},
    {a="%-?(aN)(Y)", b="ى"},
    {a="(T)%-?(aN)", b="%1"},
-   {a="(ء)%-?(aN)", b="%1"},
+   {a="([اآ])(ء)%-?(aN)", b="%1%2"}, --new
    {a="([^TA])%-?(aN)", b="%1ا"},
    {a="%-?(iNI?)", b=""},
    -- initial straight double quote gives a connective ʾalif. This has
