@@ -2,7 +2,7 @@
 This file is part of the `arabluatex' package
 
 ArabLuaTeX -- Processing ArabTeX notation under LuaLaTeX
-Copyright (C) 2016  Robert Alessi
+Copyright (C) 2016--2017  Robert Alessi
 
 Please send error reports and suggestions for improvements to Robert
 Alessi <alessi@robertalessi.net>
@@ -161,6 +161,7 @@ buckwalter = {
    {a="F", b="aN"},
    {a="K", b="iN"},
    {a="o", b="\""},
+   {a="P", b="o"}, -- pass on to \arbnull
    -- hard-coded connective alif
    {a="%[", b="ٱ"},
    {a="L", b="ٱ"} -- BW safe
@@ -169,6 +170,12 @@ buckwalter = {
 hamza = {
    -- next line for ʾiʿrāb hyphen
    {a="(')(%-)([uaiUAI])", b="%1%3"},
+   -- next lines for ʾalif alone
+   {a="(%.A)([uai]?)l%-(%^n)", b="ا%2ل%3"}, --additional (^n is lunar)
+   {a="([%(%[%|%<%s%-o])(%.A)([uai]?)l%-(%^n)", b="%1ا%3%4"}, --additional (^n is lunar) --p
+   {a="(%.A)([uai]?)l%-([%_%^%.]?[tdrzsnc])", b="ا%2ل%3%3"},
+   {a="([%(%[%|%<%s%-o])(%.A)([uai]?)l%-([%_%^%.]?[tdrzsnc])", b="%1ا%3ل%4%4"}, --p
+   {a="%.A", b="ا"},
    -- hard coded hamza
    {a="|\"'", b="ء"},
    {a="A\"'", b="آ"},
@@ -187,67 +194,89 @@ hamza = {
    {a="%'%_I", b="إI"},
    -- taḫfīfu 'l-hamza
    {a="'u'([^uaiUAI])", b="أU%1"},
-   {a="'i'([^uaiUAI])", b="إI%1"},   
+   {a="'i'([^uaiUAI])", b="إI%1"},
+   {a="([wf]a)%-\'([^uaiUAIl][^%-])", b="%1أْ%2"},
+   {a="^u'([^uaiUAI])", b="اU%1"},
+   {a="([^uaiUAIYN][%s%(%[%<])u'([^uaiUAI])", b="%1اU%2"},
+   {a="^i'([^uaiUAI])", b="اI%1"},
+   {a="([^uaiUAIYN][%s%(%[%<])i'([^uaiUAI])", b="%1اI%2"},
    -- madda (historic writing below)
    {a="'a'([^uaiUAI])", b="آ%1"},
-   {a="([^uiUI])\'a?A([%_%^%.]?[%`%'btjghxdrzsfqklmnywAY])", b="%1آ%2"},
+   {a="([^uiyUI])\'a?A([%_%^%.]?[%`%'btjghxdrzsfqklmnywAY])", b="%1آ%2"},
    {a="^\'a?A([%_%^%.]?[%`%'btjghxdrzsfqklmnywAY])", b="آ%1"},
    {a="(%W)\'a?A([%_%^%.]?[%`%'btjghxdrzsfqklmnywAY])", b="%1آ%2"},
-   {a="(A)(')(uN?)$", b="aآء%3"},
-   {a="(A)(')(uN?)(%W)", b="aآء%3%4"},
-   {a="(A)(')(iN?)$", b="aآء%3"},
-   {a="(A)(')(iN?)(%W)", b="aآء%3%4"},
+   {a="(A)(')(uN?%p?)$", b="aآء%3"},
+   {a="(A)(')(uN?)(%p?%s)", b="aآء%3%4"},
+   {a="(A)(')(iN?%p?)$", b="aآء%3"},
+   {a="(A)(')(iN?)(%p?%s)", b="aآء%3%4"},
    {a="(A)(')([iI])", b="aآئ%3"}, -- historic madda
    {a="(A)(')(u)", b="aآؤ%3"}, -- historic madda
    {a="(A)(')", b="aآء"}, -- historic madda
    -- initial (needs both ^ and %W patterns)
+   -- 'aw: the diphthong is to be resolved into 'awi' (next 8 lines)
+   {a="^('aw)(o)('[%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1i"},
+   {a="(%W)('aw)(o)('[%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1%2i"},
+   {a="^('aw)(o)(\"?[uai])([%S]-o)", b="%1i"},
+   {a="(%W)('aw)(o)(\"?[uai])([%S]-o)", b="%1%2i"},
+   {a="^('aw)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1i%2%3"},
+   {a="(%W)('aw)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1%2i%3%4"},
+   {a="^('aw)(%s)([%(%[%|%<]?\"?[uai])", b="%1i%2%3"}, --p
+   {a="(%W)('aw)(%s)([%(%[%|%<]?\"?[uai])", b="%1%2i%3%4"}, --p
+   -- then the 'initial' rules for the remaining cases
    {a="^(')([ua])", b="أ%2"},
    {a="^(')(i)", b="إ%2"},
-   {a="(%W)(')([ua])", b="%1أ%3"},
-   {a="(%W)(')(i)", b="%1إ%3"},
+   -- consider replacing initial %W with [%s%(%[%<%-]:
+   --   {a="(%W)(')([ua])", b="%1أ%3"},
+   --   {a="(%W)(')(i)", b="%1إ%3"},
+   {a="([%s%(%[%<%-])(')([ua])", b="%1أ%3"},
+   {a="([%s%(%[%<%-])(')(i)", b="%1إ%3"},
    -- final
-   -- ^say'aN and .zim'aN are special orthographies
-   {a="(%^say)(%')(aN)", b="%1ئ%3"},
-   {a="(.zi?m)(%')(aN)", b="%1ئ%3"},
-   {a="([^uai])(')([uai]N?)$", b="%1ء%3"},
-   {a="([^uai])(')([uai]N?)(%W)", b="%1ء%3%4"},
+   -- mi'aT is special orthography (unlike ^say'aN and .zim'aN):
+   -- {a="(%^sa%.?[yY])(\"?%|?)(%')(aN)", b="%1%2ئ%4"}, --new
+   -- {a="(.zi?m)(%')(aN)", b="%1ئ%3"}, --new
+   {a="(mi)(%')(a[Tt])", b="%1ائ%3"},
+   {a="(mi)(%')(aN%_?[AY])", b="%1أ%3"},
+   -- final hamzah is on the line after a letter of prolongation or a
+   -- consonant with sukūn
+   {a="([^Auai])(')(\"?[uai]N?)(%p?)$", b="%1ء%3%4"}, --new
+   {a="([^Auai])(')(\"?[uai]N?)(%p?%s)", b="%1ء%3%4"},
 -- u
-   {a="(u)(')([uai]N?)$", b="%1ؤ%3"},
-   {a="(u)(')([uai]N?)(%W)", b="%1ؤ%3%4"},
-   {a="(u)(')$", b="%1ؤ"},
-   {a="(u)(')(%W)", b="%1ؤ%3"},
+   {a="(u)(')([uai]N?%p?)$", b="%1ؤ%3"},
+   {a="(u)(')([uai]N?)(%p?%s)", b="%1ؤ%3%4"},
+   {a="(u)(')(%p?)$", b="%1ؤ%3"},
+   {a="(u)(')(%p?%s)", b="%1ؤ%3"},
 -- a
-   {a="(a)(')(A)$", b="%1آ"},
-   {a="(a)(')(A)(%W)", b="%1آ%4"},
-   {a="(a)(')([u]N?)$", b="%1أ%3"},
-   {a="(a)(')([u]N?)(%W)", b="%1أ%3%4"},
-   {a="(a)(')(a)$", b="%1أ%3"},
-   {a="(a)(')(a)(%W)", b="%1أ%3%4"},
-   {a="(a)(')(aN)$", b="%1أً"},
-   {a="(a)(')(aN)(%W)", b="%1أً%4"},
-   {a="(a)(')([i]N?)$", b="%1إ%3"},
-   {a="(a)(')([i]N?)(%W)", b="%1إ%3%4"},
-   {a="(a)(')$", b="%1أ"},
-   {a="(a)(')(%W)", b="%1أ%3"},
+   {a="(a)(')(A%p?)$", b="%1آ"},
+   {a="(a)(')(A)(%p?%s)", b="%1آ%4"},
+   {a="(a)(')([u]N?%p?)$", b="%1أ%3"},
+   {a="(a)(')([u]N?)(%p?%s)", b="%1أ%3%4"},
+   {a="(a)(')(a%p?)$", b="%1أ%3"},
+   {a="(a)(')(a)(%p?%s)", b="%1أ%3%4"},
+   {a="(a)(')(aN%p?)$", b="%1أً"},
+   {a="(a)(')(aN)(%p?%s)", b="%1أً%4"},
+   {a="(a)(')([i]N?%p?)$", b="%1إ%3"},
+   {a="(a)(')([i]N?)(%p?%s)", b="%1إ%3%4"},
+   {a="(a)(')(%p?)$", b="%1أ%3"},
+   {a="(a)(')(%p?%s)", b="%1أ%3"},
 -- i
-   {a="(i)(')([uai]N?)$", b="%1ئ%3"},
-   {a="(i)(')([uai]N?)(%W)", b="%1ئ%3%4"},
-   {a="(i)(')$", b="%1ئ"},
-   {a="(i)(')(%W)", b="%1ئ%3"},
+   {a="(i)(')([uai]N?%p?)$", b="%1ئ%3"},
+   {a="(i)(')([uai]N?)(%p?%s)", b="%1ئ%3%4"},
+   {a="(i)(')(%p?)$", b="%1ئ%3"},
+   {a="(i)(')(%p?%s)", b="%1ئ%3"},
 --
    -- middle
-   {a="(U)(')", b="%1ء"},
-   {a="([Iy])(')", b="%1ئ"},
+   {a="([UIwy])(')", b="%1ء"}, --new
+   -- {a="([Iy])(')", b="%1ئ"}, -- included in the above line
    {a="([^uai])(')([uU])", b="%1ؤ%3"},
-   {a="([^uai])(')([aA])", b="%1أ%3"},
+   {a="([^uai])(')(%_?[aAY])", b="%1أ%3"},
    {a="([^uai])(')([iI])", b="%1ئ%3"},
    {a="(u)(')([uU])", b="%1ؤ%3"},
-   {a="(u)(')([aA])", b="%1ؤ%3"},
+   {a="(u)(')(%_?[aAY])", b="%1ؤ%3"},
    {a="(u)(')([iI])", b="%1ئ%3"},
-   {a="(a)(')([aA])", b="%1أ%3"},
+   {a="(a)(')(%_?[aAY])", b="%1أ%3"},
    {a="(a)(')([uU])", b="%1ؤ%3"},
    {a="(a)(')([iI])", b="%1ئ%3"},
-   {a="(i)(')([aA])", b="%1ئ%3"},
+   {a="(i)(')(%_?[aAY])", b="%1ئ%3"},
    {a="(i)(')([uU])", b="%1ئ%3"},
    {a="(i)(')([iI])", b="%1ئ%3"},
    {a="(a)(')([^uaiUAI])", b="%1أ%3"},
@@ -258,6 +287,12 @@ hamza = {
 hamzaeasy = { -- differences marked below with 'easy'
    -- next line for ʾiʿrāb hyphen
    {a="(')(%-)([uaiUAI])", b="%1%3"},
+   -- next lines for ʾalif alone (easy)
+   {a="(%.A)([uai]?)l%-(%^n)", b="ا%2ل%3"}, --additional (^n is lunar)
+   {a="([%(%[%|%<%s%-o])(%.A)([uai]?)l%-(%^n)", b="%1ا%3%4"}, --additional (^n is lunar) --p
+   {a="(%.A)([uai]?)l%-([%_%^%.]?[tdrzsnc])", b="ا%2ل%3"},
+   {a="([%(%[%|%<%s%-o])(%.A)([uai]?)l%-([%_%^%.]?[tdrzsnc])", b="%1ا%3ل%4"}, --p
+   {a="%.A", b="ا"},
    -- hard coded hamza
    {a="|\"'", b="ء"},
    {a="A\"'", b="آ"},
@@ -276,69 +311,101 @@ hamzaeasy = { -- differences marked below with 'easy'
    {a="%'%_I", b="إI"},
    -- taḫfīfu 'l-hamza
    {a="'u'([^uaiUAI])", b="أU%1"},
-   {a="'i'([^uaiUAI])", b="إI%1"},   
+   {a="'i'([^uaiUAI])", b="إI%1"},
+   {a="([wf]a)%-\'([^uaiUAIl][^%-])", b="%1أْ%2"},
+   {a="^u'([^uaiUAI])", b="اU%1"},
+   {a="([^uaiUAIYN][%s%(%[%<])u'([^uaiUAI])", b="%1اU%2"},
+   {a="^i'([^uaiUAI])", b="اI%1"},
+   {a="([^uaiUAIYN][%s%(%[%<])i'([^uaiUAI])", b="%1اI%2"},
    -- madda (historic writing below)
    {a="'a'([^uaiUAI])", b="آ%1"},
-   {a="([^uiUI])\'a?A([%_%^%.]?[%`%'btjghxdrzsfqklmnywAY])", b="%1آ%2"},
+   {a="([^uiyUI])\'a?A([%_%^%.]?[%`%'btjghxdrzsfqklmnywAY])", b="%1آ%2"},
    {a="^\'a?A([%_%^%.]?[%`%'btjghxdrzsfqklmnywAY])", b="آ%1"},
    {a="(%W)\'a?A([%_%^%.]?[%`%'btjghxdrzsfqklmnywAY])", b="%1آ%2"},
    --easy (begin)
-   {a="(A)(')(uN?)$", b="aاء%3"},
-   {a="(A)(')(uN?)(%W)", b="aاء%3%4"},
-   {a="(A)(')(iN?)$", b="aاء%3"},
-   {a="(A)(')(iN?)(%W)", b="aاء%3%4"},
+   {a="(A)(')(uN?%p?)$", b="aاء%3"},
+   {a="(A)(')(uN?)(%p?%s)", b="aاء%3%4"},
+   {a="(A)(')(iN?%p?)$", b="aاء%3"},
+   {a="(A)(')(iN?)(%p?%s)", b="aاء%3%4"},
    {a="(A)(')([iI])", b="aائ%3"}, -- historic madda
    {a="(A)(')(u)", b="aاؤ%3"}, -- historic madda
    {a="(A)(')", b="aاء"}, -- historic madda
    --easy (end)
    -- initial (needs both ^ and %W patterns)
+   -- 'aw: the diphthong is to be resolved into 'awi' (next 8 lines)
+   {a="^('aw)(o)('[%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1i"},
+   {a="(%W)('aw)(o)('[%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1%2i"},
+   {a="^('aw)(o)(\"?[uai])([%S]-o)", b="%1i"},
+   {a="(%W)('aw)(o)(\"?[uai])([%S]-o)", b="%1%2i"},
+   {a="^('aw)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1i%2%3"},
+   {a="(%W)('aw)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1%2i%3%4"},
+   {a="^('aw)(%s)([%(%[%|%<]?\"?[uai])", b="%1i%2%3"},
+   {a="(%W)('aw)(%s)([%(%[%|%<]?\"?[uai])", b="%1%2i%3%4"},
+   -- then the 'initial' rules for the remaining cases
    {a="^(')([ua])", b="أ%2"},
    {a="^(')(i)", b="إ%2"},
-   {a="(%W)(')([ua])", b="%1أ%3"},
-   {a="(%W)(')(i)", b="%1إ%3"},
+   -- consider replacing initial %W with [%s%(%[%<%-]:
+   --   {a="(%W)(')([ua])", b="%1أ%3"},
+   --   {a="(%W)(')(i)", b="%1إ%3"},
+   {a="([%s%(%[%<%-])(')([ua])", b="%1أ%3"},
+   {a="([%s%(%[%<%-])(')(i)", b="%1إ%3"},
    -- final
-   -- ^say'aN and .zim'aN are special orthographies
-   {a="(%^say)(%')(aN)", b="%1ئ%3"},
-   {a="(.zi?m)(%')(aN)", b="%1ئ%3"},
-   {a="([^uai])(')([uai]N?)$", b="%1ء%3"},
-   {a="([^uai])(')([uai]N?)(%W)", b="%1ء%3%4"},
+   -- mi'aT is special orthography (unlike ^say'aN and .zim'aN)
+   -- {a="(%^sa%.?[yY])(\"?%|?)(%')(aN)", b="%1%2ئ%4"}, --new
+   -- {a="(.zi?m)(%')(aN)", b="%1ئ%3"}, --new
+   {a="(mi)(%')(a[Tt])", b="%1ائ%3"},
+   {a="(mi)(%')(aN%_?[AY])", b="%1أ%3"},
+   -- easy (begin)
+   -- The Munjid says that such words as radI'aN do not have the
+   -- hamzah alone on the line, so take out the following two lines
+   -- (final hamzah is on the line after a letter of prolongation or a
+   -- consonant with sukūn)
+   -- {a="([^Auai])(')(\"?[uai]N?)(%p?)$", b="%1ء%3%4"}, --new
+   -- {a="([^Auai])(')(\"?[uai]N?)(%p?%s)", b="%1ء%3%4"},
+   {a="([^Auai])(')(\"?aN)(%p?)$", b="%1ئ%3%4"}, --new
+   {a="([^Auai])(')(\"?aN)(%p?%s)", b="%1ئ%3%4"}, --new
+   {a="([^uai])(')(\"?a)(%p?)$", b="%1ء%3%4"}, --new
+   {a="([^uai])(')(\"?a)(%p?%s)", b="%1ء%3%4"}, --new
+   {a="([^uai])(')(\"?[ui]N?)(%p?)$", b="%1ء%3%4"}, --new
+   {a="([^uai])(')(\"?[ui]N?)(%p?%s)", b="%1ء%3%4"}, --new
+   --easy (end)
 -- u
-   {a="(u)(')([uai]N?)$", b="%1ؤ%3"},
-   {a="(u)(')([uai]N?)(%W)", b="%1ؤ%3%4"},
-   {a="(u)(')$", b="%1ؤ"},
-   {a="(u)(')(%W)", b="%1ؤ%3"},
+   {a="(u)(')([uai]N?%p?)$", b="%1ؤ%3"},
+   {a="(u)(')([uai]N?)(%p?%s)", b="%1ؤ%3%4"},
+   {a="(u)(')(%p?)$", b="%1ؤ%3"},
+   {a="(u)(')(%p?%s)", b="%1ؤ%3"},
 -- a
-   {a="(a)(')(A)$", b="%1آ"},
-   {a="(a)(')(A)(%W)", b="%1آ%4"},
-   {a="(a)(')([u]N?)$", b="%1أ%3"},
-   {a="(a)(')([u]N?)(%W)", b="%1أ%3%4"},
-   {a="(a)(')(a)$", b="%1أ%3"},
-   {a="(a)(')(a)(%W)", b="%1أ%3%4"},
-   {a="(a)(')(aN)$", b="%1أً"},
-   {a="(a)(')(aN)(%W)", b="%1أً%4"},
-   {a="(a)(')([i]N?)$", b="%1إ%3"},
-   {a="(a)(')([i]N?)(%W)", b="%1إ%3%4"},
-   {a="(a)(')$", b="%1أ"},
-   {a="(a)(')(%W)", b="%1أ%3"},
+   {a="(a)(')(A%p?)$", b="%1آ"},
+   {a="(a)(')(A)(%p?%s)", b="%1آ%4"},
+   {a="(a)(')([u]N?%p?)$", b="%1أ%3"},
+   {a="(a)(')([u]N?)(%p?%s)", b="%1أ%3%4"},
+   {a="(a)(')(a%p?)$", b="%1أ%3"},
+   {a="(a)(')(a)(%p?%s)", b="%1أ%3%4"},
+   {a="(a)(')(aN%p?)$", b="%1أً"},
+   {a="(a)(')(aN)(%p?%s)", b="%1أً%4"},
+   {a="(a)(')([i]N?%p?)$", b="%1إ%3"},
+   {a="(a)(')([i]N?)(%p?%s)", b="%1إ%3%4"},
+   {a="(a)(')(%p?)$", b="%1أ%3"},
+   {a="(a)(')(%p?%s)", b="%1أ%3"},
 -- i
-   {a="(i)(')([uai]N?)$", b="%1ئ%3"},
-   {a="(i)(')([uai]N?)(%W)", b="%1ئ%3%4"},
-   {a="(i)(')$", b="%1ئ"},
-   {a="(i)(')(%W)", b="%1ئ%3"},
+   {a="(i)(')([uai]N?%p?)$", b="%1ئ%3"},
+   {a="(i)(')([uai]N?)(%p?%s)", b="%1ئ%3%4"},
+   {a="(i)(')(%p?)$", b="%1ئ%3"},
+   {a="(i)(')(%p?%s)", b="%1ئ%3"},
 --
    -- middle
-   {a="(U)(')", b="%1ء"},
-   {a="([Iy])(')", b="%1ئ"},
+   {a="([Uw])(')", b="%1ء"}, --new
+   {a="([Iy])(')", b="%1ئ"}, --easy
    {a="([^uai])(')([uU])", b="%1ؤ%3"},
-   {a="([^uai])(')([aA])", b="%1أ%3"},
+   {a="([^uai])(')(%_?[aAY])", b="%1أ%3"},
    {a="([^uai])(')([iI])", b="%1ئ%3"},
    {a="(u)(')([uU])", b="%1ؤ%3"},
-   {a="(u)(')([aA])", b="%1ؤ%3"},
+   {a="(u)(')(%_?[aAY])", b="%1ؤ%3"},
    {a="(u)(')([iI])", b="%1ئ%3"},
-   {a="(a)(')([aA])", b="%1أ%3"},
+   {a="(a)(')(%_?[aAY])", b="%1أ%3"},
    {a="(a)(')([uU])", b="%1ؤ%3"},
    {a="(a)(')([iI])", b="%1ئ%3"},
-   {a="(i)(')([aA])", b="%1ئ%3"},
+   {a="(i)(')(%_?[aAY])", b="%1ئ%3"},
    {a="(i)(')([uU])", b="%1ئ%3"},
    {a="(i)(')([iI])", b="%1ئ%3"},
    {a="(a)(')([^uaiUAI])", b="%1أ%3"},
@@ -347,25 +414,34 @@ hamzaeasy = { -- differences marked below with 'easy'
 }
 
 tanwin = {
+   -- assimilations (begin)
+   {a="(o[%S]-)(%-?[uai]N[UI]?)(o)([rlmnwy])", b="%4%4"},
+   {a="(%-?[uai]NU)(%s)([rlmnwy])", b="%1%2%3%3"},
+   -- assimilations (end)
+   {a="(o[%S]-)(%-?[uai]N[UI]?)(o)([uai])", b="%4"},
    {a="%-?uNU", b="ٌو"},
    {a="%-?aNU", b="ًوا"},
    {a="%-?iNU", b="ٍو"},
    -- assimilations (begin)
    {a="%-?(uN)(%s)([rlmnwy])", b="ٌ%2%3%3"},
+   {a="(o[%S]-)(%-?aN)(_A)(o)([rlmnwy])", b="%5%5"},
+   {a="(o[%S]-)(%-?aN)(Y)(o)([rlmnwy])", b="%5%5"},
    {a="%-?(aN)(_A)(%s)([rlmnwy])", b="ًى%3%4%4"},
    {a="%-?(aN)(Y)(%s)([rlmnwy])", b="ًى%3%4%4"},
    {a="(T)%-?(aN)(%s)([rlmnwy])", b="%1ً%3%4%4"},
-   {a="(ء)%-?(aN)(%s)([rlmnwy])", b="%1ً%3%4%4"},
+   {a="(ء)%-?(aN)(%s)([rlmnwy])", b="%1%2%3%4%4"}, --new
    {a="([^TA])%-?(aN)(%s)([rlmnwy])", b="%1ًا%3%4%4"},
    {a="%-?(iNI?)(%s)([rlmnwy])", b="ٍ%2%3%3"},
    -- assimilations (end)
+   {a="(o[%S]-)(%-?aN)(_A)(o)([uai])", b="%5"},
+   {a="(o[%S]-)(%-?aN)(Y)(o)([uai])", b="%5"},
    -- quoted tanwīn (begin)
    {a="%-?(\"uN)", b=""},
    {a="(B)%-?(\"aN)", b="%1"},
    {a="%-?(\"aN)(_A)", b="ى"},
    {a="%-?(\"aN)(Y)", b="ى"},
    {a="(T)%-?(\"aN)", b="%1"},
-   {a="(ء)%-?(\"aN)", b="%1"},
+   {a="([اآ])(ء)%-?(\"aN)", b="%1%2"}, --new
    {a="([^TA])%-?(\"aN)", b="%1ا"},
    {a="%-?(\"iNI?)", b=""},
    -- quoted tanwīn (end)
@@ -374,31 +450,40 @@ tanwin = {
    {a="%-?(aN)(_A)", b="ًى"},
    {a="%-?(aN)(Y)", b="ًى"},
    {a="(T)%-?(aN)", b="%1ً"},
-   {a="(ء)%-?(aN)", b="%1ً"},
+   {a="([اآ])(ء)%-?(aN)", b="%1%2ً"}, --new
    {a="([^TA])%-?(aN)", b="%1ًا"},
    {a="%-?(iNI?)", b="ٍ"}
 }
 
 tanwineasy = { -- 'easy' requires some lines to be taken out:
+   -- assimilations (begin)
+--   {a="(o[%S]-)(%-?[uai]N[UI]?)(o)([rlmnwy])", b="%4%4"},
+--   {a="(%-?[uai]NU)(%s)([rlmnwy])", b="%1%2%3%3"},
+   -- assimilations (end)
+   {a="(o[%S]-)(%-?[uai]N[UI]?)(o)([uai])", b="%4"},
    {a="%-?uNU", b="ٌو"},
    {a="%-?aNU", b="ًوا"},
    {a="%-?iNU", b="ٍو"},
    -- assimilations (begin)
 --   {a="%-?(uN)(%s)([rlmnwy])", b="ٌ%2%3%3"},
+--   {a="(o[%S]-)(%-?aN)(_A)(o)([rlmnwy])", b="%5%5"},
+--   {a="(o[%S]-)(%-?aN)(Y)(o)([rlmnwy])", b="%5%5"},
 --   {a="%-?(aN)(_A)(%s)([rlmnwy])", b="ًى%3%4%4"},
 --   {a="%-?(aN)(Y)(%s)([rlmnwy])", b="ًى%3%4%4"},
 --   {a="(T)%-?(aN)(%s)([rlmnwy])", b="%1ً%3%4%4"},
---   {a="(ء)%-?(aN)(%s)([rlmnwy])", b="%1ً%3%4%4"},
+--   {a="(ء)%-?(aN)(%s)([rlmnwy])", b="%1%2%3%4%4"}, --new
 --   {a="([^TA])%-?(aN)(%s)([rlmnwy])", b="%1ًا%3%4%4"},
 --   {a="%-?(iNI?)(%s)([rlmnwy])", b="ٍ%2%3%3"},
    -- assimilations (end)
+   {a="(o[%S]-)(%-?aN)(_A)(o)([uai])", b="%5"},
+   {a="(o[%S]-)(%-?aN)(Y)(o)([uai])", b="%5"},
    -- quoted tanwīn (begin)
    {a="%-?(\"uN)", b=""},
    {a="(B)%-?(\"aN)", b="%1"},
    {a="%-?(\"aN)(_A)", b="ى"},
    {a="%-?(\"aN)(Y)", b="ى"},
    {a="(T)%-?(\"aN)", b="%1"},
-   {a="(ء)%-?(\"aN)", b="%1"},
+   {a="([اآ])(ء)%-?(\"aN)", b="%1%2"}, --new
    {a="([^TA])%-?(\"aN)", b="%1ا"},
    {a="%-?(\"iNI?)", b=""},
    -- quoted tanwīn (end)
@@ -407,7 +492,7 @@ tanwineasy = { -- 'easy' requires some lines to be taken out:
    {a="%-?(aN)(_A)", b="ًى"},
    {a="%-?(aN)(Y)", b="ًى"},
    {a="(T)%-?(aN)", b="%1ً"},
-   {a="(ء)%-?(aN)", b="%1ً"},
+   {a="([اآ])(ء)%-?(aN)", b="%1%2ً"}, --new
    {a="([^TA])%-?(aN)", b="%1ًا"},
    {a="%-?(iNI?)", b="ٍ"}
 }
@@ -415,61 +500,89 @@ tanwineasy = { -- 'easy' requires some lines to be taken out:
 trigraphs = { -- trigraphs or more
    -- 'llatI / 'llad_I
    {a="^'ll(a)([%_]?[dt])", b="الّ%1%2"},
-   {a="([%s%-])'ll(a)([%_]?[dt])", b="%1الّ%2%3"},
+   {a="([%(%[%|%<%s%-])'ll(a)([%_]?[dt])", b="%1الّ%2%3"}, --p
+   -- law: the diphthong is to be resolved into 'awi' (next 8 lines)
+   {a="^(law)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1i"},
+   {a="(%W)(law)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1%2i"},
+   {a="^(law)(o)(\"?[uai])([%S]-o)", b="%1i"},
+   {a="(%W)(law)(o)(\"?[uai])([%S]-o)", b="%1%2i"},
+   {a="^(law)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1i%2%3"},
+   {a="(%W)(law)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1%2i%3%4"},
+   {a="^(law)(%s)([%(%[%|%<]?\"?[uai])", b="%1i%2%3"}, --p
+   {a="(%W)(law)(%s)([%(%[%|%<]?\"?[uai])", b="%1%2i%3%4"}, --p
    -- al- + lām
    {a="^(a)l%-(l)", b="ا%1ل%2%2"},
-   {a="([%s%-])(a)l%-(l)", b="%1ا%2ل%3%3"},
-   -- al- + solar consonant
-   {a="^(a)l%-([%_%^%.]?[tdrzsn])", b="ا%1ل%2%2"},
-   {a="([%s%-])(a)l%-([%_%^%.]?[tdrzsn])", b="%1ا%2ل%3%3"},
-   -- assim. art. + solar consonant
-   {a="^(a)([%_%^%.]?[tdrzsn])%-", b="ا%1ل%2"},
-   {a="([%s%-])(a)([%_%^%.]?[tdrzsn])%-", b="%1ا%2ل%3"},
+   {a="([%(%[%|%<%s%-o])(a)l%-(l)", b="%1ا%2ل%3%3"}, --p
+   -- al- + solar consonant ('c' and '^n' are additional characters)
+   {a="^(a)l%-(%^n)", b="ا%1ل%2"}, -- ^n is lunar
+   {a="([%(%[%|%<%s%-o])(a)l%-(%^n)", b="%1ا%2ل%3"},-- ^n is lunar --p
+   {a="^(a)l%-([%_%^%.]?[tdrzsnc])", b="ا%1ل%2%2"},
+   {a="([%(%[%|%<%s%-o])(a)l%-([%_%^%.]?[tdrzsnc])", b="%1ا%2ل%3%3"}, --p
+   -- assim. art. + solar consonant ('c' and '^n' are additional characters)
+   {a="^(a)(%^n)%-", b="ا%1ل"}, -- ^n is lunar
+   {a="([%(%[%|%<%s%-o])(a)(%^n)%-", b="%1ا%2ل"},-- ^n is lunar --p
+   {a="^(a)([%_%^%.]?[tdrzsnc])%-", b="ا%1ل%2"},
+   {a="([%(%[%|%<%s%-o])(a)([%_%^%.]?[tdrzsnc])%-", b="%1ا%2ل%3"}, --p
    -- al- + initial unstable hamza
    {a="^(a)l%-(\")([uai])", b="ا%1ل%3ٱ"},
-   {a="([%s%-])(a)l%-(\")([uai])", b="%1ا%2ل%4ٱ"},
+   {a="([%(%[%|%<%s%-o])(a)l%-(\")([uai])", b="%1ا%2ل%4ٱ"}, --p
    {a="^(a)l%-([uai])", b="ا%1ل%2ا"},
-   {a="([%s%-])(a)l%-([uai])", b="%1ا%2ل%3ا"},
+   {a="([%(%[%|%<%s%-o])(a)l%-([uai])", b="%1ا%2ل%3ا"}, --p
    -- li-/la- + art. + initial unstable hamza is a special orthography
    {a="l([ai])%-l%-(\")([uai])", b="ل%1ل%3ٱ"},
    {a="l([ai])%-l%-([uai])", b="ل%1ل%2ا"},
    -- al- + lunar consonant (i.e. what remains)
    {a="^(a)l%-", b="ا%1ل"},
-   {a="([%s%-])(a)l%-", b="%1ا%2ل"},
+   {a="([%(%[%|%<%s%-o])(a)l%-", b="%1ا%2ل"}, --p
    -- diphthongs to be resolved before ʾalif conjunctionis
+   {a="(aW)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="awuا"},
+   {a="(aw)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1u"},
+   {a="(ay)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1i"},
    {a="(aW)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="awuا%2%3"},
    {a="(aw)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1u%2%3"},
    {a="(ay)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1i%2%3"},
    -- art. with waṣla + lām
    {a="'l%-(l)", b="ال%1%1"},
    -- art. with waṣla + solar consonant
-   {a="'l%-([%_%^%.]?[tdrzsn])", b="ال%1%1"},
+   -- ('c' and '^n' are additional characters)
+   {a="'l%-(%^n)", b="ال%1"}, -- ^n is lunar
+   {a="'l%-([%_%^%.]?[tdrzsnc])", b="ال%1%1"},
    -- li-/la- + art. + lām
    {a="l([ai])%-l%-(l)", b="ل%1%2%2"},
-   -- assim. art. with waṣla + solar consonant
-   {a="'([%_%^%.]?[tdrzsn])%-", b="ال%1"},
+   -- assim. art. with waṣla + solar consonant ('c' and '^n' are
+   -- additional characters)   
+   {a="'(%^n)%-", b="ال"}, -- ^n is lunar
+   {a="'([%_%^%.]?[tdrzsnc])%-", b="ال%1"},
    -- li-/la- + art. + solar consonant is a special orthography
-   {a="l([ai])%-l%-([%_%^%.]?[tdrzsn])", b="ل%1ل%2%2"},
+   -- ('c' and '^n' are additional characters)
+   {a="l([ai])%-l%-(%^n)", b="ل%1ل%2"}, -- '^n' is lunar
+   {a="l([ai])%-l%-([%_%^%.]?[tdrzsnc])", b="ل%1ل%2%2"},
    -- li-/la + assim. art. + solar consonant is a special orthography
-   {a="l([ai])%-([%_%^%.]?[tdrzsn])%-([%_%^%.]?[tdrzsn])", b="ل%1ل%3%3"},
+   -- ('c' and '^n' are additional characters)
+   {a="l([ai])%-(%^n)%-(%^n)", b="ل%1ل%3"}, -- ^n is lunar
+   {a="l([ai])%-([%_%^%.]?[tdrzsnc])%-([%_%^%.]?[tdrzsnc])", b="ل%1ل%3%3"},
    -- art. with waṣla + initial unstable hamza
    {a="'l%-(\")([uai])", b="ال%2ٱ"},
    {a="'l%-([uai])", b="ال%1ا"},   
    -- art. with waṣla + lunar consonant (i.e. what remains)
    {a="'l%-", b="ال"},
    -- the silent wāw
-   {a="uU$", b="uو"},
-   {a="uU(%W)", b="uو%1"},
-   {a="aU$", b="aو"},
-   {a="aU(%W)", b="aو%1"},
-   {a="iU$", b="iو"},
-   {a="iU(%W)", b="iو%1"},
+   {a="uU(%p?)$", b="uو%1"},
+   {a="uU(%p?%s)", b="uو%1"},
+   {a="aU(%p?)$", b="aو%1"},
+   {a="aU(%p?%s)", b="aو%1"},
+   {a="iU(%p?)$", b="iو%1"},
+   {a="iU(%p?%s)", b="iو%1"},
    -- words ending in -āT with silent wāw/yāʾ
    {a="(_a)UA", b="%1وا"},
    {a="(_a)U", b="%1و"},
-   {a="(_a)I", b="%1ي"},
+   {a="(_a)I", b="%1ي"}
+}
+
+idgham = {
    -- assimilations
-   {a="(n)(%s)([rlmnwy])", b="%1%2%3%3"}
+   {a="(n)(%s)([rlmnwy])", b="%1%2%3%3"},
+   {a="(n)(o)([rlmnwy])([%S]-o)", b="%3"}
 }
 
 trigraphseasy = { -- differences marked below with 'easy'
@@ -477,61 +590,84 @@ trigraphseasy = { -- differences marked below with 'easy'
    {a="l%-l_ah", b="l-ll_ah"},
    -- 'llatI / 'llad_I
    {a="^'ll(a)([%_]?[dt])", b="الّ%1%2"},
-   {a="([%s%-])'ll(a)([%_]?[dt])", b="%1الّ%2%3"},
+   {a="([%(%[%|%<%s%-])'ll(a)([%_]?[dt])", b="%1الّ%2%3"}, --p
+   -- law: the diphthong is to be resloved into 'awi' (next 8 lines)
+   {a="^(law)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1i"},
+   {a="(%W)(law)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1%2i"},
+   {a="^(law)(o)(\"?[uai])([%S]-o)", b="%1i"},
+   {a="(%W)(law)(o)(\"?[uai])([%S]-o)", b="%1%2i"},
+   {a="^(law)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1i%2%3"},
+   {a="(%W)(law)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1%2i%3%4"},
+   {a="^(law)(%s)([%(%[%|%<]?\"?[uai])", b="%1i%2%3"}, --p
+   {a="(%W)(law)(%s)([%(%[%|%<]?\"?[uai])", b="%1%2i%3%4"}, --p
    -- al- + lām (easy)
    {a="^(a)l%-(l)", b="ا%1ل%2"},
-   {a="([%s%-])(a)l%-(l)", b="%1ا%2ل%3"},
-   -- al- + solar consonant (easy)
-   {a="^(a)l%-([%_%^%.]?[tdrzsn])", b="ا%1ل%2"},
-   {a="([%s%-])(a)l%-([%_%^%.]?[tdrzsn])", b="%1ا%2ل%3"},
-   -- assim. art. + solar consonant (easy)
-   {a="^(a)([%_%^%.]?[tdrzsn])%-", b="ا%1ل"},
-   {a="([%s%-])(a)([%_%^%.]?[tdrzsn])%-", b="%1ا%2ل"},
+   {a="([%(%[%|%<%s%-o])(a)l%-(l)", b="%1ا%2ل%3"}, --p
+   -- al- + solar consonant (easy) ('c' and '^n' are additional characters)
+   {a="^(a)l%-(%^n)", b="ا%1ل%2"}, -- ^n is lunar
+   {a="([%(%[%|%<%s%-o])(a)l%-(%^n)", b="%1ا%2ل%3"}, -- ^n is lunar --p
+   {a="^(a)l%-([%_%^%.]?[tdrzsnc])", b="ا%1ل%2"},
+   {a="([%(%[%|%<%s%-o])(a)l%-([%_%^%.]?[tdrzsnc])", b="%1ا%2ل%3"}, --p
+   -- assim. art. + solar consonant (easy) ('c' and '^n' are
+   -- additional characters)
+   {a="^(a)(%^n)%-", b="ا%1ل"}, -- ^n is lunar
+   {a="([%(%[%|%<%s%-o])(a)(%^n)%-", b="%1ا%2ل"}, -- ^n is lunar --p
+   {a="^(a)([%_%^%.]?[tdrzsnc])%-", b="ا%1ل"},
+   {a="([%(%[%|%<%s%-o])(a)([%_%^%.]?[tdrzsnc])%-", b="%1ا%2ل"}, --p
    -- al- + initial unstable hamza
    {a="^(a)l%-(\")([uai])", b="ا%1ل%3ٱ"},
-   {a="([%s%-])(a)l%-(\")([uai])", b="%1ا%2ل%4ٱ"},
+   {a="([%(%[%|%<%s%-o])(a)l%-(\")([uai])", b="%1ا%2ل%4ٱ"}, --p
    {a="^(a)l%-([uai])", b="ا%1ل%2ا"},
-   {a="([%s%-])(a)l%-([uai])", b="%1ا%2ل%3ا"},
+   {a="([%(%[%|%<%s%-o])(a)l%-([uai])", b="%1ا%2ل%3ا"}, --p
    -- li-/la- + art. + initial unstable hamza is a special orthography
    {a="l([ai])%-l%-(\")([uai])", b="ل%1ل%3ٱ"},
    {a="l([ai])%-l%-([uai])", b="ل%1ل%2ا"},
    -- al- + lunar consonant (i.e. what remains)
    {a="^(a)l%-", b="ا%1ل"},
-   {a="([%s%-])(a)l%-", b="%1ا%2ل"},
+   {a="([%(%[%|%<%s%-o])(a)l%-", b="%1ا%2ل"}, --p
    -- diphthongs to be resolved before ʾalif conjunctionis
+   {a="(aW)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="awuا"},
+   {a="(aw)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1u"},
+   {a="(ay)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1i"},
    {a="(aW)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="awuا%2%3"},
    {a="(aw)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1u%2%3"},
    {a="(ay)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1i%2%3"},
    -- art. with waṣla + lām (easy)
    {a="'l%-(l)", b="ال%1"},
    -- art. with waṣla + solar consonant (easy)
-   {a="'l%-([%_%^%.]?[tdrzsn])", b="ال%1"},
+   -- ('c' and '^n' are additional characters)
+   {a="'l%-(%^n)", b="ال%1"}, -- ^n is lunar
+   {a="'l%-([%_%^%.]?[tdrzsnc])", b="ال%1"},
    -- li-/la- + art. + lām (easy)
    {a="l([ai])%-l%-(l)", b="ل%1%2"},
    -- assim. art. with waṣla + solar consonant (easy)
-   {a="'([%_%^%.]?[tdrzsn])%-", b="ال"},
+   -- ('c' and '^n' are additional characters)
+   {a="'(%^n)%-", b="ال"}, -- ^n is lunar
+   {a="'([%_%^%.]?[tdrzsnc])%-", b="ال"},
    -- li-/la- + art. + solar consonant is a special orthography (easy)
-   {a="l([ai])%-l%-([%_%^%.]?[tdrzsn])", b="ل%1ل%2"},
+   -- ('c' and '^n' are additional characters)
+   {a="l([ai])%-l%-(%^n)", b="ل%1ل%2"}, -- ^n is lunar
+   {a="l([ai])%-l%-([%_%^%.]?[tdrzsnc])", b="ل%1ل%2"},
    -- li-/la + assim. art. + solar consonant is a special orthography (easy)
-   {a="l([ai])%-([%_%^%.]?[tdrzsn])%-([%_%^%.]?[tdrzsn])", b="ل%1ل%3"},
+   -- ('c' and '^n' are additional characters)
+   {a="l([ai])%-(%^n)%-(%^n)", b="ل%1ل%3"}, -- ^n is lunar
+   {a="l([ai])%-([%_%^%.]?[tdrzsnc])%-([%_%^%.]?[tdrzsnc])", b="ل%1ل%3"},
    -- art. with waṣla + initial unstable hamza
    {a="'l%-(\")([uai])", b="ال%2ٱ"},
    {a="'l%-([uai])", b="ال%1ا"},
    -- art. with waṣla + lunar consonant (i.e. what remains)
    {a="'l%-", b="ال"},
    -- the silent wāw
-   {a="uU$", b="uو"},
-   {a="uU(%W)", b="uو%1"},
-   {a="aU$", b="aو"},
-   {a="aU(%W)", b="aو%1"},
-   {a="iU$", b="iو"},
-   {a="iU(%W)", b="iو%1"},
+   {a="uU(%p?)$", b="uو%1"},
+   {a="uU(%p?%s)", b="uو%1"},
+   {a="aU(%p?)$", b="aو%1"},
+   {a="aU(%p?%s)", b="aو%1"},
+   {a="iU(%p?)$", b="iو%1"},
+   {a="iU(%p?%s)", b="iو%1"},
    -- words ending in -āT with silent wāw/yāʾ
    {a="(_a)UA", b="%1وا"},
    {a="(_a)U", b="%1و"},
-   {a="(_a)I", b="%1ي"},
-   -- assimilations
---easy   {a="(n)(%s)([rlmnwy])", b="%1%2%3%3"}
+   {a="(_a)I", b="%1ي"}
 }
 
 digraphs = {
@@ -549,23 +685,35 @@ digraphs = {
    -- ʾiʿrāb (end)
    -- initial straight double quote gives a connective ʾalif
    {a="^\"[uai]", b="ٱ"},
-   {a="([%s%-])\"[uai]", b="%1ٱ"},
-   {a="(aW)(%s)([uai])", b="awuا%2%3"},
+   {a="([%(%[%|%<%s%-])\"[uai]", b="%1ٱ"}, --p
+   -- diphthongs to be resolved before ʾalif conjunctionis
+   {a="(aW)(o)(\"?[uai])([%S]-o)", b="awuا"},
+   {a="(aW)(%s)([%(%[%|%<]?)([uai])", b="awuا%2%3%4"}, --p
+   {a="(aw)(o)(\"?[uai])([%S]-o)", b="%1u"},
+   {a="(aw)(%s)([%(%[%|%<]?)(\"?[uai])", b="%1u%2%3ا"}, --p
+   {a="(ay)(o)(\"?[uai])([%S]-o)", b="%1i"},
+   {a="(ay)(%s)([%(%[%|%<]?)(\"?[uai])", b="%1i%2%3ا"}, --p
    -- hyphen + initial alif without hamza:
-   {a="(%-)([uai])([%^%_%.%`]?)(%a)", b="%1ا%3%4"},
-   {a="^([uai])", b="ا%1"},      -- initial alif without hamza
-   {a="(%s)([uai])", b="%1ا"}, -- initial alif without hamza
+   {a="([uai]%-)(\"?[uai])([%^%_%.%`]?)([%aإأؤئ])", b="%1ا%3%4"},
+   -- initial alif without hamza
+   {a="^([%(%[%|%<]?)(\"?[uai])", b="%1ا%2"}, --p
+   -- initial alif without hamza
+   {a="(o[%S]-)([uaiUAIY])(o)(\"?[uai])", b="ا"},
+   {a="(%s)([%(%[%|%<]?)(\"?[uai])", b="%1%2ا"}, --p
    {a="%-%-", b="ـ"},
    {a="ؤؤ", b="ؤّ"},
    {a="أأ", b="أّ"},
    {a="ئئ", b="ئّ"},
    {a="bb", b="بّ"},
    {a="BB", b="ـّ"},
-   {a="(%_)([thd])([thd])", b="%1%2|%3"},
+   {a="([%_%^%.])([tghdsz])([tghdsz])", b="%1%2|%3"},
+   -- same as above for additional characters:
+   {a="([%_%^%.])([cn])([cn])", b="%1%2|%3"},
    {a="tt", b="تّ"},
    {a="%_t%_t", b="ثّ"},
    {a="jj", b="جّ"},
    {a="%^g%^g", b="جّ"},
+   {a="%.h%.h", b="حّ"},
    {a="xx", b="خّ"},
    {a="%_h%_h", b="خّ"},
    {a="dd", b="دّ"},
@@ -589,6 +737,15 @@ digraphs = {
    {a="hh", b="هّ"},
    {a="ww", b="وّ"},
    {a="yy", b="يّ"},
+   {a="%.y%.y", b="ىّ"},
+   -- additional characters + šaddah (begin)
+   {a="pp", b="پّ"},
+   {a="vv", b="ڤّ"},
+   {a="gg", b="گّ"},
+   {a="%^c%^c", b="چّ"},
+   {a="%^z%^z", b="ژّ"},
+   {a="%^n%^n", b="ڭّ"},
+   -- additional characters + šaddah (end)
    {a="_t", b="ث"},
    {a="%^g", b="ج"},
    {a="%.h", b="ح"},
@@ -600,6 +757,12 @@ digraphs = {
    {a="%.t", b="ط"},
    {a="%.z", b="ظ"},
    {a="%.g", b="غ"},
+   {a="%.y", b="ى"},
+   -- additional characters (begin)
+   {a="%^c", b="چ"},
+   {a="%^z", b="ژ"},
+   {a="%^n", b="ڭ"},
+   -- additional characters (end)
    {a="(U)(A)", b="%1ا"},
    {a="WA", b="وا"},
    {a="(a)W\"", b="%1وْا"},
@@ -637,6 +800,11 @@ single = {
    {a="w", b="و"},
    {a="y", b="ي"},
    {a="T", b="ة"},
+   -- additional characters (begin)
+   {a="p", b="پ"},
+   {a="v", b="ڤ"},
+   {a="g", b="گ"},
+   -- additional characters (end)
    {a="\"$", b="ْ"},
    {a="\"(%W)", b="ْ%1"},
    {a="\"([^uaiUAI])", b="ْ%1"},
@@ -652,6 +820,8 @@ longv = {
    {a="A", b="َا"},
    {a="U", b="ُو"},
    {a="I", b="ِي"},
+   {a="aY", b="aى"},
+   {a="iY", b="iى"},
    {a="Y", b="aى"}
 }
 
@@ -680,12 +850,18 @@ punctuation = {
    {a="%+%@%[", b="]"},
    {a="%-%@%[", b="["},
    {a="%.", b="."},
-   {a="([^0-9])%,", b="%1،"},
+   -- replaced with the next two rules to make the Arabic comma work
+   --   after \abraces{}
+--   {a="([^0-9])%,", b="%1،"},
+   {a="%,", b="،"},
+   {a="([%d])%،", b="%1,"},
    {a="%?", b="؟"},
    {a="%;", b="؛"},
 }
 
 null = {
    {a="%|", b=""},
-   {a="([^0-9])(%-)", b="%1"}
+   {a="([^0-9])(%-)", b="%1"},
+   {a="o[%S]-o", b=""},
+   {a="[%^%_](.)", b=">??<%1"}
 }
